#!/bin/sh

if [ ! -f /usr/bin/format_hdd_flag ] ; then
	echo "Format service skip"
	exit 0
fi

rm /usr/bin/format_hdd_flag

LOG="/usr/bin/logger -t $0"

WARNING="${LOG} Warning: "
ERROR="${LOG} Error: "

TXTCONFIG=/usr/bin/txtconfig
TRAL_CONF_DIR="/etc/"
LABEL=tral-storage
DISK_TYPE_DIR=/dev/disk/type/hdd

DISK_BASENAME=`echo ${DISK_TYPE_DIR}/sd? | awk '{ print substr($NF,length($NF)-2,3) }'`
DISK=/dev/${DISK_BASENAME}
PART1=${DISK}1
PART2=${DISK}2

echo "Format service start for $PART1"
if [ ! -b ${DISK} ] ; then
	${ERROR} "You haven't a (HDD) disk"
	echo "You haven't a (HDD) disk"
	exit 1
fi

if [ "$1" != "automount" ] ; then 
	/usr/bin/savermgr.sh stop
	udevadm control --stop_exec_queue
fi

mount | grep "${PART1}" 1>/dev/null 2>/dev/null
if [ $? -eq 0 ] ; then
	/bin/umount -f "${PART1}"
	if [ $? -ne 0 ] ; then
		/bin/umount -l "${PART1}"
		if [ $? -ne 0 ] ; then
			${ERROR} "Unable to unmount ${PART1}"
			exit 1
		fi
	fi
fi

/usr/bin/wdctl 1 4000

echo -n "Preparing media..." ; dd if=/dev/zero of=${DISK} bs=1024 count=8192 2>/dev/null ; echo " done"
${LOG} "Partition ${PART1}"
/sbin/fdisk ${DISK} <<'EOT' 1>/dev/null 2>/dev/null
o
n
p
2

+200M
t
82
n
p
1


w
EOT

if [ $? -ne 0 ] ; then
	${ERROR} "Unable to fdisk ${DISK}"
	echo "Unable to fdisk ${DISK}"
	exit 1
fi

/sbin/mkswap ${PART2} 1>/dev/null 2> /dev/null
/sbin/swapon ${PART2} 1>/dev/null 2> /dev/null

${LOG} "Formatting storage ${PART1}"
echo "Formatting ${PART1}"
/sbin/mke2fs -q -O dir_index,sparse_super,has_journal -i 1048576 -j -m 0 ${PART1} -L "$LABEL" 1> /dev/null 2> /dev/null
if [ $? -ne 0 ] ; then
	${ERROR} "Unable to format ${PART1}"
	echo "Unable to format ${PART1}"
fi

/sbin/swapoff ${PART2} 1>/dev/null 2> /dev/null


if [ "$1" != "automount" ] ; then 
	udevadm control --start_exec_queue
	/usr/bin/savermgr.sh start
fi

echo "Format service done"

exit 0
