#!/bin/sh

DEFAULTCFGFILE=${TRAL_CONF_DIR}/streams.ini
TEMPCFGFILE=/tmp/adjust_video.ini
SETCMD="killall -USR1 video_manager"
RESETCMD="killall -USR1 video_manager"
TIMEOUT=120
RUNFLAG=/tmp/adjust_video.pid

myusage() {

	echo "Usage: $0 < reset | set | try | help >"

}

myhelp() {

	myusage

	echo -e "Adjust video. This script set video settings (brightness, contrast, saturation, hue)."
	echo -e "\thelp    - print this help"
	echo -e "\treset   - reset video settings to defaults"
	echo -e "\tset     - set video settings from ${DEFAULTCFGFILE}"
	echo -e "\ttry     - try to set video settings from ${TEMPCFGFILE}. This settings will be cancel after ${TIMEOUT} seconds."
	echo -e "\t          <config file> format:"
	echo -e "\t            [ stream_<N> ] "
	echo -e "\t              brightness = <val>"
	echo -e "\t              contrast = <val>"
	echo -e "\t              saturation = <val>"
	echo -e "\t              hue = <val>"
	echo -e "\t          where <val> = [0..255]"

}

myreset() {

	rm ${TEMPCFGFILE}
	${RESETCMD}

}

myset() {

	${SETCMD}

}

mystopdaemon() {
	rm -f ${RUNFLAG}
}

on_sigusr() {
	myset
	SECONDS=0
} 

mydaemon() {
	echo $$ > ${RUNFLAG}
	trap on_sigusr USR1

	SECONDS=0
	while [ ${SECONDS} -le ${TIMEOUT} ] ; do
		if [ ! -f ${RUNFLAG} ] ; then
			break;
		fi
		sleep 1
		let SECONDS=$SECONDS+1
	done

	if [ -f ${RUNFLAG} ] ; then
		myreset
		rm -f ${RUNFLAG}
	fi
}

case "$1" in
  help|-h)
  	myhelp
  	;;
  try)
	if [ -f ${RUNFLAG} ] ; then
		DPID=`cat ${RUNFLAG}`
		kill -USR1 $DPID
		exit
	else
		myset
		# demonization
		nice -n 20 $0 _daemon_ 1>/dev/null 2>/dev/null &
	fi
	;;
  _daemon_)
	mydaemon
	;;
  set)
	mystopdaemon
  	myset
  	;;
  reset)
	mystopdaemon
  	myreset
  	;;
  *)
	myusage
	;;
esac
