#!/bin/sh

################################
# mount manager script.
# Copyright (C) 2009 by alphara
# aaar@mail.ru
################################

# include profile (need for getting ${TRAL_CONF_DIR})
. /etc/profile

if [ -z "${TRAL_CONF_DIR}" ] ; then
	TRAL_CONF_DIR="/etc/"
fi

# --- Input arguments --- #
ARG1=$1
ARG2=$2
DISCSECTION=$ARG2

# --- Standart utilities --- #
SED=/bin/sed
AWK=/usr/bin/awk
GREP=/bin/grep
CUT=/usr/bin/cut
ECHO=/bin/echo
CAT=/bin/cat
HEAD=/usr/bin/head 
TAIL=/usr/bin/tail
SORT=/usr/bin/sort
WC=/usr/bin/wc

MOUNT=/bin/mount
UMOUNT=/bin/umount
MKDIR=/bin/mkdir
TOUCH=/bin/touch
RM=/bin/rm
SYNC=/bin/sync
LN=/bin/ln
LS=/bin/ls

NICE=/bin/nice
PS=/bin/ps
KILL=/bin/kill
KILLALL=/usr/bin/killall

SEQ=/usr/bin/seq
SLEEP=/bin/sleep
IFCONFIG=/sbin/ifconfig
TXTCONFIG=/usr/bin/txtconfig
MD5SUM=/usr/bin/md5sum

LOG="/usr/bin/logger -t $0"
LOGREAD=/sbin/logread

# --- Debug --- #

# no debug
DEBUG="/bin/true"
# debug to LOG
#DEBUG="${LOG} Debug: "

# warnings
WARNING="${LOG} Warning: "
# errors
ERROR="${LOG} Error: "
WRITECHECKFILE=write.check
REALMOUNTPOINT=/mnt/tmpmount

#################################
# Print usage
myusage() {

	${ECHO} "Usage: $0 < help | mount | unmount | remount > <disc section>"

}


#################################
# Print help
myhelp() {

	myusage

	${ECHO} -e "Mount manager. This script manage mounting storages."
	${ECHO} -e "\thelp    - print this help"
	${ECHO} -e "\tmount   - mount storage <disc section>"
	${ECHO} -e "\tunmount - unmount storage <disc section>"
	${ECHO} -e "\tremount - remount storage <disc section>"

}


#################################
# Mount storage
mymount() {		

	if [ -z "${DISCSECTION}" ] ; then
		${ERROR} "Parameter <disc section> not defined."
		exit 1
	fi
			
	ON=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} ON`
	if [ $? -ne 0 ] ; then
		${ERROR} "No \"ON\" parameter in [ ${DISCSECTION} ]."
		exit 1
	fi
		
	if [ ${ON} -ne 1 ] ; then

		${ERROR} "Disc ${DISCSECTION} disabled."
		exit 1
		
	fi

	MOUNTPOINT=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} mountpoint | ${SED} 's/\/*$//g'`
	MOUNTED=0			
	if [ -n "${MOUNTPOINT}" ] ; then
		${DEBUG} "${MOUNT} | ${GREP} \\"${MOUNTPOINT}\\" 1>/dev/null 2>/dev/null"
		${MOUNT} | ${GREP} "${MOUNTPOINT}" 1>/dev/null 2>/dev/null
		if [ $? -eq 0 ] ; then
			${DEBUG} "${MOUNTPOINT} already mounted"
			MOUNTED=1
		else
			if [ ! -d ${MOUNTPOINT} ] ; then
				${MKDIR} -p ${MOUNTPOINT} 1>/dev/null 2>/dev/null
			fi			
			if [ ! -d ${REALMOUNTPOINT} ] ; then
				${MKDIR} -p ${REALMOUNTPOINT} 1>/dev/null 2>/dev/null
			fi

			URL=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} url`
			if [ -n "${URL}" ] ; then
				# URL parsing: get ${PROTO}://[${USER}[:${PASS}]@]${PATH}
				PROTO=`${ECHO} ${URL} | ${SED} 's/:\\/\\/.*//g'`
				PATH=`${ECHO} ${URL} | ${SED} 's/.*:\\/\\///g'`
				USER=""
				PASS=""
				${ECHO} ${PATH} | ${GREP} "@" 1>/dev/null 2>/dev/null
				if [ $? -eq 0 ] ; then
					USER=`${ECHO} ${PATH} | ${CUT} -d @ -f 1`
					${ECHO} ${USER} | ${GREP} ":" 1>/dev/null 2>/dev/null
					if [ $? -eq 0 ] ; then	
						PASS=`${ECHO} ${USER} | ${CUT} -d : -f 2`
						USER=`${ECHO} ${USER} | ${CUT} -d : -f 1`
					fi
					PATH=`${ECHO} ${PATH} | ${CUT} -d @ -f 2`
				fi

				# mounting storage
				if [ "${PROTO}" = "local" ] ; then
					# local dir
					${DEBUG} "local dir ${MOUNTPOINT}"
				elif [ "${PROTO}" = "smb" ] ; then
					# smb
					SMBPATH=`${ECHO} ${PATH} | ${SED} 's/\\//\\\\/g'`
					if [ -n "${USER}" ] ; then
						OPTIONS="user=${USER}"
						if [ -n "${PASS}" ] ; then
							OPTIONS="${OPTIONS},pass=${PASS}"
						fi
					else
						OPTIONS="user=guest"
					fi
								
					MAINMOUNTDIR=`${ECHO} ${SMBPATH} | ${CUT} -d \\\\ -f 1-2`
					
					REALMOUNTED=0
					${DEBUG} "${MOUNT} | ${GREP} \\"${REALMOUNTPOINT}\\" 1>/dev/null 2>/dev/null"
					${MOUNT} | ${GREP} "${REALMOUNTPOINT}" 1>/dev/null 2>/dev/null
					if [ $? -eq 0 ] ; then
						${DEBUG} "${REALMOUNTPOINT} already mounted"
						REALMOUNTED=1
					else
						${DEBUG} "${MOUNT} -t cifs -o ${OPTIONS} \\\\${MAINMOUNTDIR} ${REALMOUNTPOINT}"
						${MOUNT} -t cifs -o ${OPTIONS} \\\\${MAINMOUNTDIR} ${REALMOUNTPOINT}
						if [ $? -eq 0 ] ; then
							REALMOUNTED=1
						fi
					fi
								
					if [ ${REALMOUNTED} -eq 1 ] ; then
						MOUNTED=1			
						PATH=`${ECHO} ${PATH} | ${CUT} -d / -f 3-`
						${DEBUG} "PATH = ${PATH}"
						if [ ! -d ${REALMOUNTPOINT}/${PATH} ] ; then
							${MKDIR} -p ${REALMOUNTPOINT}/${PATH} 1>/dev/null 2>/dev/null
							if [ $? -ne 0 ] ; then
								${ERROR} "No access to mkdir ${PATH} on ${MAINMOUNTDIR} mounted on ${REALMOUNTPOINT}"
								MOUNTED=0
							fi
						fi
						
						${DEBUG} "${TOUCH} ${REALMOUNTPOINT}/${WRITECHECKFILE} 2>/dev/null 1>/dev/null"
						${TOUCH} ${REALMOUNTPOINT}/${WRITECHECKFILE} 2>/dev/null 1>/dev/null
						if [ $? -ne 0 ] ; then
							${ERROR} "No access to write on ${MAINMOUNTDIR} mounted on ${REALMOUNTPOINT}"
							MOUNTED=0
						fi
						
						if [ ${MOUNTED} -eq 1 ] ; then
							${DEBUG} "${MOUNT} -o bind ${REALMOUNTPOINT}/${PATH} ${MOUNTPOINT}"
							${MOUNT} -o bind ${REALMOUNTPOINT}/${PATH} ${MOUNTPOINT}
							
							${DEBUG} "${UMOUNT} ${REALMOUNTPOINT}"
							${UMOUNT} ${REALMOUNTPOINT}
							if [ $? -ne 0 ] ; then
								${ERROR} "Not unmounted ${REALMOUNTPOINT}"
							fi
						else
							${DEBUG} "${UMOUNT} ${REALMOUNTPOINT}"
							${UMOUNT} ${REALMOUNTPOINT}
							if [ $? -ne 0 ] ; then
								${ERROR} "Not unmounted ${REALMOUNTPOINT}"
							fi
							exit 1
						fi
	
					else
						${ERROR} "URL ${URL} is not mount"
						exit 1
					fi
				elif [ "${PROTO}" = "nfs" ] ; then
					# nfs
					MAINMOUNTDIR=`${ECHO} ${PATH} | ${CUT} -d / -f 1-2`
					
					${DEBUG} "${MOUNT} -o nolock,soft ${MAINMOUNTDIR} ${REALMOUNTPOINT}"
					${MOUNT} -o nolock,soft ${MAINMOUNTDIR} ${REALMOUNTPOINT}
					if [ $? -eq 0 ] ; then
						${TOUCH} ${REALMOUNTPOINT}/${WRITECHECKFILE} 2>/dev/null 1>/dev/null
						if [ $? -ne 0 ] ; then
							${ERROR} "No access to write on ${REALMOUNTPOINT}"
						else
							MOUNTED=1
					
							PATH=`${ECHO} ${PATH} | ${CUT} -d / -f 3-`
							${DEBUG} "PATH = ${PATH}"
							if [ ! -d ${REALMOUNTPOINT}/${PATH} ] ; then
								${MKDIR} -p ${REALMOUNTPOINT}/${PATH} 1>/dev/null 2>/dev/null
							fi
							${DEBUG} "${MOUNT} -o bind ${REALMOUNTPOINT}/${PATH} ${MOUNTPOINT}"
							${MOUNT} -o bind ${REALMOUNTPOINT}/${PATH} ${MOUNTPOINT}
							${DEBUG} "${UMOUNT} ${REALMOUNTPOINT}"
							${UMOUNT} ${REALMOUNTPOINT}
						fi
					else
						${ERROR} "URL ${URL} is not mount"
						exit 1
					fi
				elif [ "${PROTO}" = "usb" ] ; then
					# usb
					${DEBUG} "USB disk not mounted on ${MOUNTPOINT}"
					exit 1
				else
					${ERROR} "Incorrect URL ${URL}"
					exit 1
				fi
			fi
		fi
		
	fi

}

#################################
# Unmount storage
myunmount() {

	if [ -z "${DISCSECTION}" ] ; then
		${ERROR} "Parameter <disc section> not defined."
		exit 1
	fi
			
	${SYNC}
	${SYNC}
	
	ON=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} ON`
	if [ ${ON} -eq 1 ] ; then

		MOUNTPOINT=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} mountpoint | ${SED} 's/\/*$//g'`
		if [ -n "${MOUNTPOINT}" ] ; then
			${MOUNT} | ${GREP} "${MOUNTPOINT}" 1>/dev/null 2>/dev/null
			if [ $? -eq 0 ] ; then

				URL=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} url`
				if [ -n "${URL}" ] ; then
					# URL parsing: get ${PROTO}://
					PROTO=`${ECHO} ${URL} | ${SED} 's/:\\/\\/.*//g'`
					${DEBUG} "PROTO = ${PROTO}"

					# unmounting storage
					if [ "${PROTO}" = "local" ] ; then
						# local dir
						# nothing
						${DEBUG} "local"
					elif [ "${PROTO}" = "smb" ] ; then
						# smb
						${UMOUNT} ${MOUNTPOINT}
					elif [ "${PROTO}" = "nfs" ] ; then
						# nfs
						${UMOUNT} ${MOUNTPOINT}
					elif [ "${PROTO}" = "usb" ] ; then
						# usb
						# nothing
						exit 1
					else
						${ERROR} "Incorrect URL ${URL}"
						exit 1
					fi
				fi
			fi
		fi
	fi
	
	${DEBUG} "${MOUNT} | ${GREP} '/tmp/tmpmount' 1>/dev/null 2>/dev/null"
	${MOUNT} | ${GREP} "/tmp/tmpmount" 1>/dev/null 2>/dev/null
	if [ $? -eq 0 ] ; then
		${UMOUNT} /tmp/tmpmount
	fi	
}


case "$1" in
  help|-h)
  	myhelp
  	;;
  start|mount)
	mymount
	;;
  stop|unmount)
  	myunmount
  	;;
  restart|reload|remount)                                   
  	myunmount
	mymount
	;;
  *)
	myusage
	;;
esac
