#!/bin/sh

################################
# saver manager script.
# Copyright (C) 2009 by alphara
# aaar@mail.ru
################################
# Last modified 2016.03.07
################################

# --- Modes --- #

# Usb secure mode
# USBSECUREMODE=<0|1>,
# 0=disable: access to usb independent from access to other storages.
# 1=enable: if Tral has access to other storages, then access to usb will be denied.
USBSECUREMODE=0

# Usb disc flag mode
# USBDISCFLAGMODE=<0|1>,
# 0=disable: access to usb defined by command "mount"
# 1=enable: access to usb defined by exist file ${USBDISCFLAG}
USBDISCFLAGMODE=1

# include profile (need for getting ${TRAL_CONF_DIR})
. /etc/profile

if [ -z "${TRAL_CONF_DIR}" ] ; then
	TRAL_CONF_DIR="/etc/"
fi

# --- Input arguments --- #
ARG1=$1
ARG2=$2

# --- Standart utilities --- #
SED=/bin/sed
AWK=/usr/bin/awk
GREP=/bin/grep
CUT=/usr/bin/cut
ECHO=/bin/echo
CAT=/bin/cat
HEAD=/usr/bin/head
TAIL=/usr/bin/tail
SORT=/usr/bin/sort
WC=/usr/bin/wc
EXPR=/usr/bin/expr

MMOUNT="/usr/bin/mountmgr.sh mount"
MUMOUNT="/usr/bin/mountmgr.sh unmount"
UMOUNT=/bin/umount
MOUNT=/bin/mount
MKDIR=/bin/mkdir
TOUCH=/bin/touch
RM=/bin/rm
SYNC=/bin/sync
LN=/bin/ln
LS=/bin/ls
DF=/bin/df
UDEVADM=/sbin/udevadm

NICE=/bin/nice
PS=/bin/ps
KILL=/bin/kill
KILLALL=/usr/bin/killall

SEQ=/usr/bin/seq
SLEEP=/bin/sleep
IFCONFIG=/sbin/ifconfig
CHECKSTATE="/usr/bin/stmgrclient check"
RISEEVENT="/usr/bin/stmgrclient rise"
FAILEVENT="/usr/bin/stmgrclient fail"
TXTCONFIG=/usr/bin/txtconfig
SETTINGS=/usr/bin/settings.sh
MD5SUM=/usr/bin/md5sum

LOG="/usr/bin/logger -t $0"
LOGREAD=/sbin/logread

# --- Debug --- #

# no debug
DEBUG="/bin/true"
# debug to LOG
#DEBUG="${LOG} Debug: "

# warnings
WARNING="${LOG} Warning: "
# errors
ERROR="${LOG} Error: "

# --- Global variables --- #
RUNFLAG=/tmp/savermgr-run.flag
SLEEPFLAG=/tmp/savermgr-sleep.flag
USBDISCFLAG=/tmp/savermgr-usbdisc.flag
NOSDCARDFLAG=/tmp/savermgr-nosdcard.flag
NOSDCARDTIME=/tmp/savermgr-nosdcard.time
CURRENTTIMESEC="/usr/bin/lctime -f %s"
NOSDCARDTIMESEC="/bin/cat ${NOSDCARDTIME}"
NOSDCARDWAITSEC=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini savermgr sdcard_poll_interval`
#mmc#USBDISCREMOVEFLAG=/tmp/mmc.getout.flag
WAITINGSTOPFLAG=/tmp/savermgr-waitingstop.flag
WRITECHECKFILE=write.check
NEEDSOFT=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini savermgr needsoft`
# ${SECTIONS} has established order (for usb secure mode "usbdisc" must be last!!!):
SECTIONS=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini savermgr sections`
DIRPARAMS=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini savermgr dirparams`
APPLICATIONS=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini savermgr applications`
DEVICEID=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini common deviceid`
MAINDIRLINK=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini savermgr maindirlink`
REFRESHSLEEP=60
MAXLOGCOUNT=100
MAXLOGSIZE=200000
ONADDCMD=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini savermgr onaddcmd`


#################################
# Print usage
myusage() {

	${ECHO} "Usage: $0 < help | start | stop | restart | add <mountpoint> | remove | refresh >"

}


#################################
# Print help
myhelp() {

	myusage

	${ECHO} -e "Saver manager. This script manage mounting storages and running savers and cleaners."
	${ECHO} -e "\thelp    - print this help"
	${ECHO} -e "\tstart   - mount storages and run savers"
	${ECHO} -e "\tstop    - unmount storages and stop savers"
	${ECHO} -e "\trestart - remount storages and restart savers"
	${ECHO} -e "\treload  - same as restart"
	${ECHO} -e "\tadd     - run saver, that save data on usb disc"
	${ECHO} -e "\tremove  - stop saver, that save data on usb disc"
	${ECHO} -e "\trefresh - refresh all checks to run or stop savers"

}


#################################
# Init main cycle.
mystart() {

	${DEBUG} "start"

	if [ -f ${RUNFLAG} ] ; then
		${ERROR} "$0 already run"
		exit 1
	fi

	${TOUCH} ${RUNFLAG} 2>/dev/null 1>/dev/null

}


#################################
# Main cycle. Service of disc and application
mydaemon() {

	${DEBUG} "daemon"

	${TOUCH} ${SLEEPFLAG} 2>/dev/null 1>/dev/null

	MAINDIR=""

	while [ -f ${RUNFLAG} ] ; do

		${DEBUG} " "
		${DEBUG} "------------"
		${DEBUG} " "

		# for usb secure mode
		ACCESSCOUNT=0

		# check to run shared manager, state manager and videograb
		NEEDSOFTRUN=1
		for J in ${NEEDSOFT}; do
			${PS} | ${GREP} "/usr/bin/${J}" | ${GREP} -v ${GREP} 2>/dev/null 1>/dev/null
			if [ $? -ne 0 ] ; then
				NEEDSOFTRUN=0
				${WARNING} "Waiting to run /usr/bin/${J}"
				break;
			fi
		done

		# saving to main directory check. Main directory uses for archive reading
		if [ -n "${MAINDIR}" ] ; then
			${TOUCH} ${MAINDIRLINK}/../${WRITECHECKFILE} 2>/dev/null 1>/dev/null
			if [ $? -ne 0 ] ; then
				MAINDIR=""
			fi
		fi

		for DISCSECTION in ${SECTIONS}; do
			${DEBUG} "--- [ ${DISCSECTION} ] ---"

			# handler of disc section
			ON=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} ON`
			if [ $? -ne 0 ] ; then
				${ERROR} "No \"ON\" parameter in [ ${DISCSECTION} ]."
				ON=0
			fi

			ACCESS=0

			MOUNTPOINT=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} mountpoint | ${SED} 's/\/*$//g'`
			${DEBUG} "MOUNTPOINT = ${MOUNTPOINT}"
			DIR=${MOUNTPOINT}/${DEVICEID}
			MOUNTED=0

			if [ ${ON} -eq 1 ] ; then
				if [ -n "${MOUNTPOINT}" ] ; then
					# mount check
					${MOUNT} | ${GREP} "${MOUNTPOINT}" 1>/dev/null 2>/dev/null
					if [ $? -eq 0 ] ; then
						MOUNTED=1

						#
						# usb disc flag mode handler
						if [ ${USBDISCFLAGMODE} -eq 1 ] ; then
							if [ "${DISCSECTION}" = "usbdisc" ] ; then
								if [ ! -f ${USBDISCFLAG} ] ; then
									MOUNTED=0
								fi
							fi
						fi
						# end of usb disc flag mode handler
						#

					else
						URL=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} url`
						case $URL in
							local*|usb*) 		
								;;
							*)		
								${WARNING} "mounting ${MOUNTPOINT}"
								${MMOUNT} ${DISCSECTION}
		
								if [ $? -eq 0 ] ; then
									${WARNING} "${MOUNTPOINT} mount - successful"
									MOUNTED=1
								else
									${WARNING} "${MOUNTPOINT} mount - failure"
								fi
								;;
						esac
					fi
				fi

				${DEBUG} "MOUNTED=${MOUNTED}"
				if [ ${MOUNTED} -eq 1 ] ; then

					# saving to storage check
					${DEBUG} "saving to storage check"
					${TOUCH} ${MOUNTPOINT}/${WRITECHECKFILE} 2>/dev/null 1>/dev/null
					if [ $? -ne 0 ] ; then
						${ERROR} "No access to write on ${REALMOUNTPOINT}"
					else
						ACCESS=1
						# directory creating (if no exist)
						${DEBUG} "directory creating"

						if [ ! -d ${DIR} ] ; then
							${MKDIR} -p ${DIR} 1>/dev/null 2>/dev/null
						fi

						for DIRPRM in ${DIRPARAMS}; do
							SUBDIR=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini ${DISCSECTION} ${DIRPRM}`
							if [ -n "${SUBDIR}" ] ; then
								if [ ! -d ${SUBDIR} ] ; then
									${DEBUG} "${MKDIR} -p ${DIR}/${SUBDIR} 1>/dev/null 2>/dev/null"
									${MKDIR} -p ${DIR}/${SUBDIR} 1>/dev/null 2>/dev/null
								fi
							fi
						done

						# define main directory (if no exist)
						if [ -z "${MAINDIR}" ] ; then
							MAINDIR=${DIR}
							${DEBUG} "MAINDIR = ${MAINDIR}"
							${RM} -f ${MAINDIRLINK}
							${LN} -sf ${MAINDIR} ${MAINDIRLINK}
							if [ $? -ne 0 ] ; then
								MAINDIR=""
								${ERROR} "Cannot create link to main directory"
							fi
						fi # if [ -z "${MAINDIR}" ] ; then
					fi #if [ $? -ne 0 ] ; then	
				else #if [ ${MOUNTED} -eq 1 ] ; then
					if [ "${DISCSECTION}" = "usbdisc" ] ; then
						ACCESS=0
					fi
				fi
			fi #if [ ${ON} -eq 1 ] ; then

			if [ ${ACCESS} -eq 1 ] ; then
				for APP in ${APPLICATIONS}; do
					${DEBUG} "run app ${APP}"
					# check: application is running?
					RUN=0
					${PS} | ${GREP} "/usr/bin/${APP} -c ${DISCSECTION}" | ${GREP} -v ${GREP} 2>/dev/null 1>/dev/null
					if [ $? -eq 0 ] ; then
						RUN=1
					fi
					if [ ${RUN} -eq 0 ] ; then
						${DEBUG} "run ${APP}"
						/usr/bin/${APP} -c ${DISCSECTION} 2>/dev/null 1>/dev/null
					fi
				done  # ~ for APP in ${APPLICATIONS}; do
			fi #if [ ${ACCESS} -eq 1 ] ; then
		done  # ~ for DISCSECTION in ${SECTIONS}; do
		${DEBUG} "--- ---"

		# test for SD card and try remount every ${NOSDCARDWAITSEC} seconds if needed
		if [ -z "$(${CAT} /proc/mounts | ${GREP} /dev/sdb1)" ] ; then
			if [ -f ${NOSDCARDFLAG} ] ; then
				if [ $(${EXPR} $(${CURRENTTIMESEC}) - $(${NOSDCARDTIMESEC})) -gt ${NOSDCARDWAITSEC} ] ; then
					${UDEVADM} trigger
					${RM} -f ${NOSDCARDFLAG} ${NOSDCARDTIME}
				fi
			else
				${TOUCH} ${NOSDCARDFLAG}
				${CURRENTTIMESEC} > ${NOSDCARDTIME}
			fi
		fi

		# sleep 60 seconds or until set ${SLEEPFLAG}
		${DEBUG} "sleep ${REFRESHSLEEP}"
		for SECONDS in `${SEQ} 1 ${REFRESHSLEEP}`; do
			if [ ! -f ${SLEEPFLAG} ] ; then
				${TOUCH} ${SLEEPFLAG}
				${DEBUG} "sleep break"
				break;
			fi
			${SLEEP} 1
		done
		${DEBUG} "sleep done"

	done  # ~ while [ -f ${RUNFLAG} ] ; do
	${RM} -f ${SLEEPFLAG} 2>/dev/null 1>/dev/null


	# kill all applications and LOG
	${DEBUG} "kill all applications and LOG"
	for APP in ${APPLICATIONS}; do
		${PS} | ${GREP} "/usr/bin/${APP}" | ${GREP} -v ${GREP} 2>/dev/null 1>/dev/null
		if [ $? -eq 0 ] ; then
			${KILLALL} ${APP}
		fi
	done

	# unmount discs
	${DEBUG} "umount"
	${SYNC}
	${SYNC}
	for DISCSECTION in ${SECTIONS}; do
		${MUMOUNT} ${DISCSECTION}
	done # ~ for DISCSECTION in ${SECTIONS}; do

	${RM} -f ${WAITINGSTOPFLAG} 2>/dev/null 1>/dev/null

	${DEBUG} "daemon done"

}


#################################
# Refresh main cycle
myrefresh() {

	${DEBUG} "refresh"
	${RM} -f ${SLEEPFLAG} 2>/dev/null 1>/dev/null

}


#################################
# Stop main cycle
mystop() {

	${DEBUG} "stop"
	MOUNTPID=`${PS} | ${GREP} mount | ${GREP} cifs | ${AWK} '{ print $1 }'`
		if [ -n ${MOUNTPID} ] ; then
		${KILL} ${MOUNTPID}
	fi

	if [ -f ${RUNFLAG} ] ; then
		${TOUCH} ${WAITINGSTOPFLAG} 2>/dev/null 1>/dev/null
		${RM} -f ${RUNFLAG} 2>/dev/null 1>/dev/null
	fi

	myrefresh

	# waiting until daemon stop
	while [ -f ${WAITINGSTOPFLAG} ] ; do
		${SLEEP} 1
	done

}


#################################
# Add usb disc
myadd() {

	${DEBUG} "add"
	MOUNTPOINT=`${ECHO} ${ARG2} | ${SED} 's/\/*$//g'`

	MP=`${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini usbdisc mountpoint | ${SED} 's/\/*$//g'`
	if [ -z "${MOUNTPOINT}" ] ; then
		MOUNTPOINT=${MP}
	else
		if [ "${MP}" != "${MOUNTPOINT}" ] ; then
			# save ${MOUNTPOINT} in config file
			${TXTCONFIG} ${TRAL_CONF_DIR}/streams.ini usbdisc mountpoint ${MOUNTPOINT} 2>/dev/null 1>/dev/null
			if [ $? -eq 0 ] ; then
				${WARNING} "${TRAL_CONF_DIR}/streams.ini changed. [ usbdisc ] mountpoint=${MOUNTPOINT}"
				${SETTINGS} save
			else
				${ERROR} "Can not change ${TRAL_CONF_DIR}/streams.ini"
				exit 1
			fi

		fi
	fi

	${WARNING} "Using [usbdisc] mountpoint=${MOUNTPOINT}"

	if [ ${USBDISCFLAGMODE} -eq 1 ] ; then
		${TOUCH} ${USBDISCFLAG} 2>/dev/null 1>/dev/null
	fi

	myrefresh

	test -n "${ONADDCMD}" && ${ONADDCMD}
}


#################################
# Remove usb disc
myremove() {

	${DEBUG} "remove"

	if [ ${USBDISCFLAGMODE} -eq 1 ] ; then
		${RM} -f ${USBDISCFLAG} 2>/dev/null 1>/dev/null
	fi

	for APP in ${APPLICATIONS}; do				
		${DEBUG} "Removing usb/mmc: kill ${APP}"
		PID=`${PS} | ${GREP} "/usr/bin/${APP} -c ${DISCSECTION}" | ${GREP} -v ${GREP} | ${AWK} '{ print($1) }'`
		if [ -z ${PID} -eq 1 ] ; then
			${KILL} ${PID}
		fi
	done  # ~ for APP in ${APPLICATIONS}; do


	myrefresh

}


#################################
# Main case
case "$1" in
  help|-h)
  	myhelp
  	;;
  start)
	# init
	mystart
	# demonization
	${NICE} -n 20 $0 _daemon_ 1>/dev/null 2>/dev/null &
	;;
  _daemon_)
	mydaemon
	;;
  stop)
  	mystop
  	;;
  restart|reload)
  	mystop
	mystart
	# demonization
	${NICE} -n 20 $0 _daemon_ 1>/dev/null 2>/dev/null &
	;;
  add)
  	myadd
	;;
  remove)
  	myremove
	;;
  refresh)
	myrefresh
	;;
  *)
	myusage
	;;
esac


# The end
#################################

