#!/bin/sh

TEMP_DIR=/tmpetc

BLOCKDEV=`cat /proc/mtd | grep etc_config | cut -f 1 -d ' ' | sed -e "s/://" -e "s:mtd:/dev/mtdblock:"`

#NAMES="sysconfig/networking/common sysconfig/networking/ifcfg-eth0 streams.ini ssmtp/revaliases resolv.conf TZ ctime safe timer.ini user.cfg"
NAMES=`cat /etc/etcconfig.names`

ETHSNUM=`ifconfig eth0 | grep HWaddr | cut -d ':' -f 5-7`

#save etc config data (if needed)
save() {

if [ $ETHSNUM == 00:00:00 ] ; then
       	echo Serial Number is 0 exiting
	exit 1
fi

if [ -z "$BLOCKDEV" ] ; then 
	logger -t $0  "save etc_config partition not exist!"
	exit 1
fi

#test for difference
if ! tar cf - /etc/ 2>/dev/null | md5sum -s -c /var/run/etc_config.md5 2>/dev/null ; then 
	
	rm -f /var/run/etc_config.md5
	if [ -b $BLOCKDEV ] ; then
	
		tar -czf $BLOCKDEV /etc 2>/dev/null
		if [ $? -ne 0 ] ; then
		
			logger -t $0 "Error: Can't save settings"
			exit 1
		fi
		#save /etc md5
		tar cf - /etc/ 2>/dev/null | md5sum > /var/run/etc_config.md5
		logger -t $0 "configuration saved"
	fi
else
	logger -t $0 "Nothing to save (md5 is not changed)"
fi

}

#mount /tmpetc and remove data from it
mount_temp(){
		if [ ! -d ${TEMP_DIR} ] ; then
			mkdir ${TEMP_DIR}
		fi

		if [ `mount | grep -c ${TEMP_DIR}` -eq 0 ] ; then
			mount tmpfs ${TEMP_DIR} -t tmpfs 
		fi

		rm -rf ${TEMP_DIR}/*
}

#load data from storage
load() {

if [ $ETHSNUM == 00:00:00 ] ; then
       	echo Serial Number is 0 exiting
	exit 1
fi
	rm -f /var/run/etc_config.md5

if [ -z "$BLOCKDEV" ] ; then 
	logger -t $0  "load etc_config partition not exist!"
	exit 1
fi

	if [ -b $BLOCKDEV ] ; then
	
		while [ `mount | grep -c /etc` -ge 1 ] ; do
			umount -l /etc
		done

		mount_temp

		tar -xzf $BLOCKDEV -C ${TEMP_DIR} 1>/dev/null 2>/dev/null
		if [ $? -ne 0 ] ; then
		
			logger -t $0 "Error: Can't load settings."
			umount -l ${TEMP_DIR}
			exit 1
		fi
		
		BROKEN=0

		for i in $NAMES ; do
		
			BNAME=/etc/${i}
			if [ ! -f ${TEMP_DIR}${BNAME} ] ; then
			
				if [ -f ${BNAME} ] ; then			
					logger -t $0 "file ${BNAME} not found."
					BROKEN=1
					break;
				fi
			fi
		done

		#check version
		C_VERSION=`cat /etc/version`
		S_VERSION=`cat ${TEMP_DIR}/etc/version`
		if [ "${C_VERSION}" != "${S_VERSION}" ] ; then
	logger -t $0 "version missmatch current ${C_VERSION} saved ${S_VERSION}"
			BROKEN=1
		fi

		if [ $BROKEN -ne 0 ] ; then
			logger -t $0 "Error: Broken settings.(names, version)"
			umount -l ${TEMP_DIR}
			exit 1
		fi
		
		mount ${TEMP_DIR}/etc /etc
		#save /etc md5
		tar cf - /etc/ 2>/dev/null | md5sum > /var/run/etc_config.md5
		logger -t $0  "configuration loaded"
	fi

}

# restore default settings
restdef() {

	rm -f /var/run/etc_config.md5
if [ -z "$BLOCKDEV" ] ; then 
	logger -t $0  "restdef etc_config partition not exist!"
	exit 1
fi

	while [ `mount | grep -c /etc` -ge 1 ] ; do
		umount -l /etc
	done

	save
	load
	logger -t $0  "configuration restored"
}

case "$1" in
    load)
  	load
	;;
    save)
  	save
	;;
    reset)
  	restdef
	;;
  *)
	echo $"Usage: $0 {load|save|reset}"
	exit 1
esac

exit $?

