#!/bin/sh

#
# First check if disk supports smart temperature.
#
PATH=/bin:/sbin:/usr/bin:/usr/sbin:$PATH

DISK_TYPE_DIR=/dev/disk/type/hdd
DISK_BASENAME=`echo ${DISK_TYPE_DIR}/sd? | awk '{ print substr($NF,length($NF)-2,3) }'`
DISKDEV=/dev/${DISK_BASENAME}

DISKMP=`txtconfig /etc/streams.ini disc0 mountpoint`

test -e "$DISKDEV" || ( echo "Error: disk device $DISKDEV is missing"; exit 1 )
test -z "$DISKMP" && ( echo "Error: streams.ini/disc0/mountpoint is not specified"; exit 1 )
mount | grep -q "$DISKMP" || ( echo "Error: disk is not mounted to $DISKMP"; exit 1 )

LOGFILE=$DISKMP/smart.log
smartctl -d ata -A $DISKDEV >/tmp/smart_full.log
STRING=`smartctl -d ata -A $DISKDEV | grep Temperature`
if [ -z "$STRING" ] ; then
	#
	# Some error occured. So any data could not be retrieved.
	#
	MAXTEMP=-1
	MAXDATE=-1
	MINTEMP=100
	MINDATE=-1
	CURTEMP=-1
	SMARTMINMAXTEMP=-1
else
	CURTEMP=`echo $STRING | awk '{ print $10 }'`
	SMARTMINMAXTEMP=`echo $STRING | sed 's/.*Lifetime Min\/Max \(.*\/.*\))/\1/g'`
	DATE=`/usr/bin/lctime -g -f "%H:%M:%S %d/%m/%Y"`

	if [ -f "$LOGFILE" ] ; then
		# load vars: MAXTEMP, MAXDATE, MINTEMP, MINDATE, CURTEMP, SMARTMINMAXTEMP
		. $LOGFILE
		# update CURTEMP
		CURTEMP=`echo $STRING | awk '{ print $10 }'`
		
		if [ "$CURTEMP" -gt "$MAXTEMP" ]; then
			MAXTEMP=$CURTEMP
			MAXDATE=$DATE
		fi

		if [ "$CURTEMP" -lt "$MINTEMP" ]; then
			MINTEMP=$CURTEMP
			MINDATE=$DATE
		fi
	else
		#
		# Write log for the first time after flash was burned.
		#
		MAXTEMP=$CURTEMP 
		MAXDATE=$DATE
		MINTEMP=$CURTEMP
		MINDATE=$DATE
	fi
fi
echo MAXTEMP=$MAXTEMP > $LOGFILE
echo MAXDATE=\"$MAXDATE\" >> $LOGFILE
echo MINTEMP=$MINTEMP >> $LOGFILE
echo MINDATE=\"$MINDATE\" >> $LOGFILE
echo CURTEMP=$CURTEMP >> $LOGFILE
echo SMARTMINMAXTEMP=\"$SMARTMINMAXTEMP\" >> $LOGFILE

cp $LOGFILE /tmp/smart.log

MSG_ALARM_TEMP="alarm temp"
MSG_ALARM_SECTORS="alarm sectors"

if [ $CURTEMP -gt 60 ] ; then
    msgsend -n 255 -a -s "$MSG_ALARM_TEMP ($CURTEMP)"
fi

if [ $CURTEMP -lt 1 ] ; then
    msgsend -n 255 -a -s "$MSG_ALARM_TEMP ($CURTEMP)"
fi

i=`smartctl -d ata -A $DISKDEV | grep 'Reallocated_Sector_Ct'`; set - $i
s=$10
if [ $s -gt 972 ] ; then
    msgsend -n 255 -a -s "$MSG_ALARM_SECTORS"
fi


MODELLIST="TOSHIBA_MK2576GSX_GS002D"

COUNT=`echo $MODELLIST | wc -w`
COUNT=`echo $COUNT`

MODEL=`hdparm -I $DISKDEV | grep 'Model Number' | awk -F ":" '{print $2}'`
MODEL=`echo $MODEL | tr ' ' '_'`
FIRMWARE=`hdparm -I $DISKDEV | grep 'Firmware Revision' | awk -F ":" '{print $2}'`
FIRMWARE=`echo $FIRMWARE | tr ' ' '_'`

DODIVIDE=0

while [ $COUNT -gt 0 ]
do
	cmd="echo $MODELLIST | awk '{print \$$COUNT}'"
	res=`sh -c "$cmd"`
	COUNT=$(($COUNT-1))
	[ "a$res" = "a${MODEL}_${FIRMWARE}" ] && DODIVIDE=1 && break
done

[ $DODIVIDE -eq 1 ] && {
	TRUEPWRONHR=`grep Power_On_Hours smart_full.log | awk '{print int($10/60)}'`
	cat /tmp/smart_full.log | sed -r "s/(Power_On_Hours.*[ ])([0-9]+)$/\1$TRUEPWRONHR/" > /tmp/smart_full.log_
	mv /tmp/smart_full.log_ /tmp/smart_full.log
}

